@@ -3,6 +3,7 @@ import unittest
 from unittest import mock
 
 from engineio import json
+from engineio import packet as eio_packet
 import pytest
 
 from socketio import exceptions
@@ -91,6 +92,7 @@ class TestServer(unittest.TestCase):
             room='room',
             skip_sid='123',
             callback='cb',
+            ignore_queue=False,
         )
         s.emit(
             'my event',
@@ -99,6 +101,7 @@ class TestServer(unittest.TestCase):
             skip_sid='123',
             namespace='/foo',
             callback='cb',
+            ignore_queue=True,
         )
         s.manager.emit.assert_called_with(
             'my event',
@@ -107,6 +110,7 @@ class TestServer(unittest.TestCase):
             room='room',
             skip_sid='123',
             callback='cb',
+            ignore_queue=True,
         )
 
     def test_emit_default_namespace(self, eio):
@@ -126,6 +130,7 @@ class TestServer(unittest.TestCase):
             room='room',
             skip_sid='123',
             callback='cb',
+            ignore_queue=False,
         )
         s.emit(
             'my event',
@@ -133,6 +138,7 @@ class TestServer(unittest.TestCase):
             room='room',
             skip_sid='123',
             callback='cb',
+            ignore_queue=True,
         )
         s.manager.emit.assert_called_with(
             'my event',
@@ -141,6 +147,7 @@ class TestServer(unittest.TestCase):
             room='room',
             skip_sid='123',
             callback='cb',
+            ignore_queue=True,
         )
 
     def test_send(self, eio):
@@ -156,9 +163,14 @@ class TestServer(unittest.TestCase):
             room='room',
             skip_sid='123',
             callback='cb',
+            ignore_queue=False,
         )
         s.send(
-            'foo', room='room', skip_sid='123', namespace='/foo', callback='cb'
+            'foo', room='room',
+            skip_sid='123',
+            namespace='/foo',
+            callback='cb',
+            ignore_queue=True,
         )
         s.manager.emit.assert_called_with(
             'message',
@@ -167,6 +179,7 @@ class TestServer(unittest.TestCase):
             room='room',
             skip_sid='123',
             callback='cb',
+            ignore_queue=True,
         )
 
     def test_call(self, eio):
@@ -259,53 +272,22 @@ class TestServer(unittest.TestCase):
             'environ', 'start_response'
         )
 
-    def test_emit_internal(self, eio):
+    def test_send_packet(self, eio):
         s = server.Server()
-        s._emit_internal('123', 'my event', 'my data', namespace='/foo')
+        s._send_packet('123', packet.Packet(
+            packet.EVENT, ['my event', 'my data'], namespace='/foo'))
         s.eio.send.assert_called_once_with(
             '123', '2/foo,["my event","my data"]'
         )
 
-    def test_emit_internal_with_tuple(self, eio):
+    def test_send_eio_packet(self, eio):
         s = server.Server()
-        s._emit_internal('123', 'my event', ('foo', 'bar'), namespace='/foo')
-        s.eio.send.assert_called_once_with(
-            '123', '2/foo,["my event","foo","bar"]'
-        )
-
-    def test_emit_internal_with_list(self, eio):
-        s = server.Server()
-        s._emit_internal('123', 'my event', ['foo', 'bar'], namespace='/foo')
-        s.eio.send.assert_called_once_with(
-            '123', '2/foo,["my event",["foo","bar"]]'
-        )
-
-    def test_emit_internal_with_none(self, eio):
-        s = server.Server()
-        s._emit_internal('123', 'my event', None, namespace='/foo')
-        s.eio.send.assert_called_once_with(
-            '123', '2/foo,["my event"]'
-        )
-
-    def test_emit_internal_with_callback(self, eio):
-        s = server.Server()
-        id = s.manager._generate_ack_id('1', 'cb')
-        s._emit_internal('123', 'my event', 'my data', namespace='/foo', id=id)
-        s.eio.send.assert_called_once_with(
-            '123', '2/foo,1["my event","my data"]'
-        )
-
-    def test_emit_internal_default_namespace(self, eio):
-        s = server.Server()
-        s._emit_internal('123', 'my event', 'my data')
-        s.eio.send.assert_called_once_with(
-            '123', '2["my event","my data"]'
-        )
-
-    def test_emit_internal_binary(self, eio):
-        s = server.Server()
-        s._emit_internal('123', u'my event', b'my binary data')
-        assert s.eio.send.call_count == 2
+        s._send_eio_packet('123', eio_packet.Packet(
+            eio_packet.MESSAGE, 'hello'))
+        assert s.eio.send_packet.call_count == 1
+        assert s.eio.send_packet.call_args_list[0][0][0] == '123'
+        pkt = s.eio.send_packet.call_args_list[0][0][1]
+        assert pkt.encode() == '4hello'
 
     def test_transport(self, eio):
         s = server.Server()
@@ -400,7 +382,6 @@ class TestServer(unittest.TestCase):
         assert s.manager.is_connected('1', '/foo')
         handler.assert_called_once_with('1', 'environ')
         s.eio.send.assert_any_call('123', '0/foo,{"sid":"1"}')
-        print(s.eio.send.call_args_list)
         s.eio.send.assert_any_call('123', '4/foo,"Unable to connect"')
 
     def test_handle_connect_always_connect(self, eio):
@@ -593,6 +574,34 @@ class TestServer(unittest.TestCase):
         catchall_handler.assert_called_once_with(
             'my message', '1', 'a', 'b', 'c')
 
+    def test_handle_event_with_catchall_namespace(self, eio):
+        s = server.Server(async_handlers=False)
+        sid_foo = s.manager.connect('123', '/foo')
+        sid_bar = s.manager.connect('123', '/bar')
+        connect_star_handler = mock.MagicMock()
+        msg_foo_handler = mock.MagicMock()
+        msg_star_handler = mock.MagicMock()
+        star_foo_handler = mock.MagicMock()
+        star_star_handler = mock.MagicMock()
+        s.on('connect', connect_star_handler, namespace='*')
+        s.on('msg', msg_foo_handler, namespace='/foo')
+        s.on('msg', msg_star_handler, namespace='*')
+        s.on('*', star_foo_handler, namespace='/foo')
+        s.on('*', star_star_handler, namespace='*')
+        s._trigger_event('connect', '/bar', sid_bar)
+        s._handle_eio_message('123', '2/foo,["msg","a","b"]')
+        s._handle_eio_message('123', '2/bar,["msg","a","b"]')
+        s._handle_eio_message('123', '2/foo,["my message","a","b","c"]')
+        s._handle_eio_message('123', '2/bar,["my message","a","b","c"]')
+        s._trigger_event('disconnect', '/bar', sid_bar)
+        connect_star_handler.assert_called_once_with('/bar', sid_bar)
+        msg_foo_handler.assert_called_once_with(sid_foo, 'a', 'b')
+        msg_star_handler.assert_called_once_with('/bar', sid_bar, 'a', 'b')
+        star_foo_handler.assert_called_once_with(
+            'my message', sid_foo, 'a', 'b', 'c')
+        star_star_handler.assert_called_once_with(
+            'my message', '/bar', sid_bar, 'a', 'b', 'c')
+
     def test_handle_event_with_disconnected_namespace(self, eio):
         s = server.Server(async_handlers=False)
         s.manager.connect('123', '/foo')
@@ -702,8 +711,10 @@ class TestServer(unittest.TestCase):
         cb = mock.MagicMock()
         id1 = s.manager._generate_ack_id('1', cb)
         id2 = s.manager._generate_ack_id('1', cb)
-        s._emit_internal('123', 'my event', ['foo'], id=id1)
-        s._emit_internal('123', 'my event', ['bar'], id=id2)
+        s._send_packet('123', packet.Packet(
+            packet.EVENT, ['my event', 'foo'], id=id1))
+        s._send_packet('123', packet.Packet(
+            packet.EVENT, ['my event', 'bar'], id=id2))
         s._handle_eio_message('123', '31["foo",2]')
         cb.assert_called_once_with('foo', 2)
 
@@ -714,7 +725,8 @@ class TestServer(unittest.TestCase):
         s._handle_eio_message('123', '0/foo,')
         cb = mock.MagicMock()
         id = s.manager._generate_ack_id('1', cb)
-        s._emit_internal('123', 'my event', ['foo'], namespace='/foo', id=id)
+        s._send_packet('123', packet.Packet(
+            packet.EVENT, ['my event', 'foo'], namespace='/foo', id=id))
         s._handle_eio_message('123', '3/foo,1["foo",2]')
         cb.assert_called_once_with('foo', 2)
 
@@ -831,6 +843,39 @@ class TestServer(unittest.TestCase):
         s.disconnect('1', '/foo')
         assert result['result'] == ('disconnect', '1')
 
+    def test_catchall_namespace_handler(self, eio):
+        result = {}
+
+        class MyNamespace(namespace.Namespace):
+            def on_connect(self, ns, sid, environ):
+                result['result'] = (sid, ns, environ)
+
+            def on_disconnect(self, ns, sid):
+                result['result'] = ('disconnect', sid, ns)
+
+            def on_foo(self, ns, sid, data):
+                result['result'] = (sid, ns, data)
+
+            def on_bar(self, ns, sid):
+                result['result'] = 'bar' + ns
+
+            def on_baz(self, ns, sid, data1, data2):
+                result['result'] = (ns, data1, data2)
+
+        s = server.Server(async_handlers=False, namespaces='*')
+        s.register_namespace(MyNamespace('*'))
+        s._handle_eio_connect('123', 'environ')
+        s._handle_eio_message('123', '0/foo,')
+        assert result['result'] == ('1', '/foo', 'environ')
+        s._handle_eio_message('123', '2/foo,["foo","a"]')
+        assert result['result'] == ('1', '/foo', 'a')
+        s._handle_eio_message('123', '2/foo,["bar"]')
+        assert result['result'] == 'bar/foo'
+        s._handle_eio_message('123', '2/foo,["baz","a","b"]')
+        assert result['result'] == ('/foo', 'a', 'b')
+        s.disconnect('1', '/foo')
+        assert result['result'] == ('disconnect', '1', '/foo')
+
     def test_bad_namespace_handler(self, eio):
         class Dummy(object):
             pass
@@ -933,6 +978,11 @@ class TestServer(unittest.TestCase):
             None,
         )
 
+    def test_shutdown(self, eio):
+        s = server.Server()
+        s.shutdown()
+        s.eio.shutdown.assert_called_once_with()
+
     def test_start_background_task(self, eio):
         s = server.Server()
         s.start_background_task('foo', 'bar', baz='baz')
